
[![Enterprise Modules](https://raw.githubusercontent.com/enterprisemodules/public_images/master/banner1.jpg)](https://www.enterprisemodules.com)

#### Table of Contents

1. [Overview](#overview)
2. [License](#license)
3. [Description - What the module does and why it is useful](#description)
4. [Setup](#setup)
  * [Requirements](#requirements)
  * [Installing the mq_install module](#installing-the-mq_install-module)
5. [Usage - Configuration options and additional functionality](#usage)
6. [Reference - An under-the-hood peek at what the module is doing and how](#reference)
7. [Limitations - OS compatibility, etc.](#limitations)

## Overview

This module allows you to install IBM MQ messaging software. It is part of our family of Puppet modules to install and manage IBM messaging middleware. Besides the [`mq_install`](https://www.enterprisemodules.com/shop/products/ibm-mq-install) module, this family also contains:

- [`mq_config`](https://www.enterprisemodules.com/shop/products/ibm-mq-config) to configure and manage the MQ software
- [`iib_config`](https://www.enterprisemodules.com/shop/products/ibm-iib-config) to manage and configure IBM Integration Bus software
- [`iib_install`](https://www.enterprisemodules.com/shop/products/ibm-iib_install) to install IBM Integration Bus software

## Try yourself

You don't want to read about it, but really want to try it? You can! You can explore our Puppet modules for Oracle by checking out our [playgrounds](https://www.enterprisemodules.com//playgrounds#mq). Here we guide you around in some of the functionality of our IBM messaging modules.

## License

This is a commercially licensed module. But you can use the module on VirtualBox based development systems for **FREE**. You can [request a **FREE** trial license here](https://www.enterprisemodules.com/sign-up-for-free-trial)

[![FREE TRIAL](https://raw.githubusercontent.com/enterprisemodules/public_images/master/free-trial.jpg)](https://www.enterprisemodules.com/sign-up-for-free-trial)

Check the [License](https://forge.puppet.com/enterprisemodules/mq_install/license) for details.

## Description

Our `mq_install` module allows you to install your IBM MQ software on your systems. It automatically configures your OS, creates required users and groups, unpacks the files and registers the license.

Here is an example on how to use this:
```puppet
include mq_install::installmq
```

Check [here](https://www.enterprisemodules.com/docs/mq_install/description.html) to see the full documentation for this module.

## Setup

### Requirements

The [`mq_install`](https://www.enterprisemodules.com/shop/products/puppet-mq_install-module) module requires:

- Puppet module [`enterprisemodules-easy_type`](https://forge.puppet.com/enterprisemodules/easy_type) installed.
- Puppet version 4.9 or higher. Can be Puppet Enterprise or Puppet Open Source
- IBM MQ 8 or 9 installation.
- A valid License from IBM to use this software.
- A valid entitlement to use this module by Enterprise Modules.
- Runs on most Linux systems, Solaris, AIX and Windows.

### Installing the mq_install module

To install these modules, you can use a `Puppetfile`

```
mod 'enterprisemodules/mq_install'               ,'x.x.x'
```

Then use the `librarian-puppet` or `r10K` to install the software.

You can also install the software using the `puppet module`  command:

```
puppet module install enterprisemodules-mq_install
```

## Usage

Building a total MQ setup requires quite some work. To help you, we have built some examples:
- [MQ only example](https://github.com/enterprisemodules/simple_mq_demo)
- [MQ and IIB example](https://github.com/enterprisemodules/simple_mqiib_demo)

These repos contain examples of how you can install and configure a basic MQ setup. 

## Reference

Here you can find some more information regarding this puppet module:

- [The MQ install documentation](https://www.enterprisemodules.com/docs/mq_install/description.html)
- [The product announcement](https://www.enterprisemodules.com/news/2017/05/IBM-modules-with-virtualsciences/)

Here is a list of related blog posts:
- [Introducing the IBM MQ Puppet module](https://www.enterprisemodules.com/blog/2017/05/introducing-mq-module/)

Some example code:
- [Simple MQ demo](https://github.com/enterprisemodules/simple_mq_demo)
- [MQ and IIB demo](https://github.com/enterprisemodules/simple_mqiib_demo)


## Limitations

This module runs most Linux versions, Solaris, AIX and Windows systems. It requires a puppet version higher than 4. Contact us of you need to run this module on other flavors of UNIX. We can work together with you to make it work.