#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# mq_install::software::unix
#
# @summary This class install's specified version of the the MQ software for Unix like systems on the system.
# This is an internal defined type and should not be used by client code.
#
# @param [String[1]] source_location
#    The location of the MQ software.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          source_location => '/software',
#          ...
#        }
#
# @param [Pattern[/\d+\.\d+\.\d+\.\d/]] version
#    The version to be installed.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          version => '9.0.0.0',
#          ...
#        }
#
# @param [String[1]] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          tmp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [String[1]] file_name
#    The file containing the install kit for the MQ software.
#
# @param [Boolean] install_ams
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_ams => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_ft
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_ft => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_client
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_client => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_explorer
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_explorer => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_xrserver
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_xserver => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_samples
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_samples => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    This component can only be installed on Linux systems.
#
# @param [Boolean] install_sdk
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_sdk => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    This component can only be installed on Linux systems.
#
# @param [Boolean] install_man
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_man => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    This component can only be installed on Linux systems.
#
# @param [Boolean] install_gskit
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_gskit => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    This component can only be installed on Linux systems.
#
# @param [Boolean] install_amqp
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_amqp => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_web
#    
#
# @param [Boolean] install_salesforce
#    Specify a value of `true` when you want to install the Bridge to Salesforce component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_salesforce => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to configure the connections to Salesforce and IBM MQ, then run the runmqsfb command to subscribe to events from Salesforce and publish them to an IBM MQ network.
#    This component can only be installed on Linux systems.
#
# @param [Boolean] install_blockchain
#    Specify a value of `true` when you want to install the IBM MQ Bridge to blockchain component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_blockchain => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to configure the connections to your blockchain network and IBM MQ. You can then run the runmqbcb command to start the bridge and send queries and updates to, and receive responses from your blockchain network. The queue manager that the bridge connects to must be an IBM MQ Advanced queue manager.
#    This component is available only on Linux for System x (64 bit), on RHEL 7.3 or later and is only supported for MQ 9.1 and higher.
#    This component can only be installed on Linux systems.
#
# @param [Boolean] install_rdqm
#    Specify a value of `true` when you want to install the replicated data queue manager component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_rqdm => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to make the replicated data queue manager high availability configuration available. See Installing RDQM (replicated data queue managers) for more information.
#    This component is available only on Linux for System x (64 bit), on RHEL 7.3 or later and is only supported for MQ 9.1 and higher.
#    This component can only be installed on Linux systems.
#
# @param [Boolean] install_toolkit
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_toolkit => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed. 
#    This component can only be installed on Windows systems.
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          logoutput => true,
#          ...
#        }
#
#--++--
define mq_install::software::unix(
  String[1]                            $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                              # Version of the MQ software to install
  String[1]                            $file_name,                            # source file to use
  String[1]                            $tmp_dir            = '/tmp',          # Directory used for unpacking
  Boolean                              $install_ams        = false,           # Install AMS packages
  Boolean                              $install_ft         = false,           # Install FT packages
  Boolean                              $install_client     = false,           # Install Client packages
  Boolean                              $install_explorer   = false,           # Install Explorer packages
  Boolean                              $install_xrserver   = false,           # Install XRServer packages
  Boolean                              $install_samples    = false,           # Install Samples packages
  Boolean                              $install_sdk        = false,           # Install SDK packages
  Boolean                              $install_man        = false,           # Install man packages
  Boolean                              $install_gskit      = false,           # Install GSKit packages
  Boolean                              $install_amqp       = false,           # Install AMQP packages
  Boolean                              $install_web        = false,           # REST API and Console
  Boolean                              $install_salesforce = false,           # IBM MQ Bridge to Salesforce
  Boolean                              $install_blockchain = false,           # IBM MQ Bridge to blockchain
  Boolean                              $install_rdqm       = false,           # replicated data queue manage
  Boolean                              $install_toolkit    = false,           # Install Development Toolkit packages
  Variant[Enum['on_failure'], Boolean] $logoutput          = 'on_failure',    # Output of the execs
)
{
  mq_install { 'mq_install::software': }

  $supported_components = lookup('mq_install::software::supported_components', Hash)
  $version_components   = $version.split('\.')
  $package_version      = "${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"

  $selected_components = mq_install::selected_components($supported_components,
      $install_ams,
      $install_ft,
      $install_client,
      $install_explorer,
      $install_xrserver,
      $install_samples,
      $install_sdk,
      $install_man,
      $install_gskit,
      $install_amqp,
      $install_web,
      $install_salesforce,
      $install_blockchain,
      $install_rdqm,
      $install_toolkit)
  #
  # If the installed version is the same as the requested version. Do nothing
  #
  $components_to_remove = $::mq_installed_components - $selected_components
  $components_to_add    = $selected_components - $::mq_installed_components

  # On SunOS we need to reinstall the software when a component is
  # removed to we need to fetch it then too
  if $::kernel == 'SunOS' {
    $fetch_software = $components_to_remove != [] or $components_to_add != []
  } else {
    $fetch_software = $components_to_add != []
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  if $components_to_remove != [] {
    #
    # MQ only allows us to uninstall a package when no components
    # are running. So we have to stop all before continuing.
    #
    class { '::mq_install::internal::mq_stop':
      logoutput => $logoutput,
    }
    contain '::mq_install::internal::mq_stop'

    if $::kernel == 'SunOS' {
      #
      # SunOS has one pkg package containing all components. The other U*X like
      # OS-es have a package for every component. So if we need to remove a component,
      # we remove everyting and reinstall only the requested componens
      #
      file { "${tmp_dir}/rmadmin":
        ensure  => 'present',
        content => 'action=nocheck',
      }
      ->exec {'remove mqm':
        command => "pkgrm -n -a ${tmp_dir}/rmadmin mqm",
        path    => '/usr/sbin',
        require => Class['::mq_install::internal::mq_stop'],
      }
    } else {
      $components_to_remove.each |$comp| {
        package { $comp:
          ensure  => 'absent',
          require => Class['::mq_install::internal::mq_stop'],
        }
      }
    }
  }

  if $fetch_software {
    #
    # When the tmp_dir is not defined, define it here.
    #
    unless defined(File[$tmp_dir]) {
      file { $tmp_dir:
        ensure => 'directory',
        before => Archive["${tmp_dir}/${file_name}"],
      }
    }
    archive { "${tmp_dir}/${file_name}":
      ensure       => present,
      extract      => true,
      extract_path => $tmp_dir,
      source       => "${source_location}/${file_name}",
      creates      => "${tmp_dir}/MQServer",
      cleanup      => true,
      require      => File[$tmp_dir],
    }
    #
    # On AIX no mqlicense is available
    #
    if $::kernel != 'AIX' {
      exec { 'Accept MQ License Requirements':
        command => "sudo ${tmp_dir}/MQServer/mqlicense.sh -accept",
        path    => '/usr/bin',
        creates => "${tmp_dir}/mq_license_${version}/license/status.dat",
        require => Archive["${tmp_dir}/${file_name}"],
      }
    }
  }
  if $::kernel == 'SunOS' {
    if $fetch_software {
      #
      # SunOS has one pkg package containing all components. The other U*X like
      # OS-es have a package for every component
      #
      file { "${tmp_dir}/answers":
        ensure  => 'present',
        content => epp('mq_install/answers.epp', {'selected_components' => $selected_components}),
      }
      -> file { '/opt/mqm':
        ensure  => 'directory',
      }
      #
      # We use an exec instead of a package because the package will not do
      # a reinstall if the selected components are changed.
      # 
      -> exec {'install mqm':
        command => "pkgadd -a ${tmp_dir}/MQServer/silent/admin -r ${tmp_dir}/answers -d ${tmp_dir}/MQServer mqm",
        path    => '/usr/sbin',
        require => Exec['Accept MQ License Requirements'],
        before  => File['Remove temporary files'],
      }
    }
  } else {
    $selected_components.each |$comp| {
      case $::kernel {
        'Linux': {
          $provider     = 'rpm'
          $source       = "${tmp_dir}/MQServer/${comp}-${package_version}.x86_64.rpm"
          $package_name = "${comp}-${package_version}"
        }
        'AIX': {
          if $comp == 'mqm.server.rte' {
            $source       = "${tmp_dir}/MQServer/mqm.server"
          } else {
            $source       = "${tmp_dir}/MQServer/${comp}"
          }
          $provider     = 'aix'
          $package_name = $comp
        }
        default: { fail "Internal error. Kernel ${::kernel} not supported."}
      }

      package {$package_name:
        ensure   => 'present',
        provider => $provider,
        source   => $source,
        before   => File['Remove temporary files'],
      }
    }
  }

  file {'Remove temporary files':
    ensure  => 'absent',
    path    => "${tmp_dir}/MQServer",
    recurse => true,
    force   => true,
  }
}
