#
# See the file "LICENSE" for the full license governing this code.
#
#
#--++--
define mq_install::software::windows(
  String                               $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                              # Version of the MQ software to install
  String                               $tmp_dir            = 'C:\\tmp',       # Directory used for unpacking
  String                               $file_name          = 'IBM_MQ_9.1_WINDOWS_TRIAL.zip', # Name of install file
  Boolean                              $install_ams        = false,           # Install AMS packages
  Boolean                              $install_ft         = false,           # Install FT packages
  Boolean                              $install_client     = false,           # Install Client packages
  Boolean                              $install_explorer   = false,           # Install Explorer packages
  Boolean                              $install_xrserver   = false,           # Install XRServer packages
  Boolean                              $install_samples    = false,           # Install Samples packages
  Boolean                              $install_sdk        = false,           # Install SDK packages
  Boolean                              $install_man        = false,           # Install man packages
  Boolean                              $install_gskit      = false,           # Install GSKit packages
  Boolean                              $install_amqp       = false,           # Install AMQP packages
  Boolean                              $install_web        = false,           # REST API and Console
  Boolean                              $install_salesforce = false,           # IBM MQ Bridge to Salesforce
  Boolean                              $install_blockchain = false,           # IBM MQ Bridge to blockchain
  Boolean                              $install_rdqm       = false,           # replicated data queue manage
  Boolean                              $install_toolkit    = false,           # Install Development Toolkit packages
  Variant[Enum['on_failure'], Boolean] $logoutput          = 'on_failure',    # Output of the execs
)
{
  mq_install { 'mq_install::software': }
  $supported_components = lookup('mq_install::software::supported_components', Hash)
  #
  # Setup some variables
  #
  $version_components = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $package_version    = "${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"

  $selected_components = mq_install::selected_components($supported_components,
      $install_ams,
      $install_ft,
      $install_client,
      $install_explorer,
      $install_xrserver,
      $install_samples,
      $install_sdk,
      $install_man,
      $install_gskit,
      $install_amqp,
      $install_web,
      $install_salesforce,
      $install_blockchain,
      $install_rdqm,
      $install_toolkit)

  $components_to_add    = $selected_components - $::mq_installed_components
  $components_to_remove = $::mq_installed_components - $selected_components
  easy_type::debug_evaluation() # Show local variable on extended debug

  if $components_to_add != [] or $components_to_remove != []{

    unless defined(File[$tmp_dir]) {
      file { $tmp_dir:
        ensure => 'directory',
      }
    }

    archive { "${tmp_dir}\\${file_name}":
      ensure       => present,
      extract      => true,
      extract_path => $tmp_dir,
      source       => "${source_location}\\${file_name}",
      creates      => "${tmp_dir}\\MQServer",
      cleanup      => true,
      require      => File[$tmp_dir],
    }

    ->file {"${tmp_dir}\\MQServer":
      ensure  => 'directory',
    }

    -> file { "${tmp_dir}\\MQServer\\Response.ini":
      ensure  => file,
      content => epp('mq_install/response.ini.epp',{'add' => $components_to_add, 'remove' =>$components_to_remove }),
      backup  => false,
    }
    -> package { 'IBM MQ(Installation1)':
      ensure          => installed,
      source          => "${tmp_dir}\\MQServer\\MSI\\IBM MQ.msi",
      install_options => ["USEINI=\"${tmp_dir}\\MQServer\\Response.ini\"", 'AGREETOLICENSE="yes"','/l*v', "${tmp_dir}\\mqserver_install.log"],
    }

    #
    # Because the mq_config module start's right away, we neede to make sure the IBM MQ task
    #  is up and running before leaving this module.
    #
    ~> exec {'wait for IBM Service manager':
      command     => epp('mq_install/wait_for_service_manager.ps1.epp'),
      refreshonly => true,
      provider    => 'powershell',
      timeout     => 60,
    }

    exec {'Remove temporary MQ files':
      path    => 'c:\\windows\\system32',
      command => "cmd.exe /c \"rmdir /s /q ${tmp_dir}\\MQServer\"",
      require => Package['IBM MQ(Installation1)'],
    }

  }
}
